@@ -849,7 +849,14 @@ meta_ui_theme_get_frame_borders (MetaUI *ui,
           gtk_style_context_set_path (style, widget_path);
           gtk_widget_path_free (widget_path);
 
-          gtk_style_context_get (style, GTK_STATE_FLAG_NORMAL, "font", &free_font_desc, NULL);
+          gtk_style_context_save (style);
+          gtk_style_context_set_state (style, GTK_STATE_FLAG_NORMAL);
+          gtk_style_context_get (style,
+                                 gtk_style_context_get_state (style),
+                                 "font",
+                                 &free_font_desc,
+                                 NULL);
+          gtk_style_context_restore (style);
           font_desc = (const PangoFontDescription *) free_font_desc;
 #else
           GtkStyle *default_style;
@@ -1097,7 +1104,18 @@ MetaUIDirection meta_ui_get_direction(void)
 	return META_UI_DIRECTION_LTR;
 }
 
-#if !GTK_CHECK_VERSION (3, 0, 0)
+#if GTK_CHECK_VERSION (3, 0, 0)
+GdkPixbuf *meta_ui_get_pixbuf_from_surface (cairo_surface_t *surface)
+{
+	gint width;
+	gint height;
+
+	width = cairo_xlib_surface_get_width (surface);
+	height = cairo_xlib_surface_get_height (surface);
+
+	return gdk_pixbuf_get_from_surface (surface, 0, 0, width, height);
+}
+#else
 GdkPixbuf* meta_ui_get_pixbuf_from_pixmap(Pixmap pmap)
 {
 	GdkPixmap* gpmap;
